function or_frq(urlr,preproc,fig_save,stroke_type)
% OR_FRQ Measurements of the local shapes orientation
% 
% Note: This is a subfunction of or_wrap.
% 
% -------------
% INPUT
% -------------
% urlr - url of text file with url of files to process
%     format: url \n url ...
% preproc   - string: 'preprocess' | {'nopreprocess'}
%           - option to preprocess the input images:
%           binarization, negative, [resizing]
% fig_save - string: 'save' | {'nosave'}
%     - saves the colorcoded orientation image
% stroke_type - string: {'outlines'} | 'inline'
%           - extract the outlines or inline of the shape
% 
% -------------
% OUTPUT
% -------------
% {url}_frq.txt - text file with the frequency count of
%     the orientation measurements, ordered 0 to 179
%     - format: filename tab angle-in-degrees tab ...
%     - save location: 'dat' directory relative to urlr file location
% *_or_cmap.png - colorcoded images of the orientations
%     - save location: 'pix' directory relative to urlr file location
% 
% -------------
% CREDITS
% -------------
% Vlad Atanasiu
% atanasiu@alum.mit.edu | http://www.waqwaq.info/atanasiu/
% 2011.08.05 - [new] activate outlines/inline option
% 2010.12.27 - creation

% parse input
if nargin < 2
    preproc = 'preprocess';
end
if nargin < 3
    fig_save = 'save';
end
if nargin < 4
    stroke_type = 'outlines';
end

% initializations
pixdir = 'pix/';
datdir = 'dat/';

% compose file fname to be created by this function
[fpath, fname] = fileparts(urlr);
% file is in the current workdirectory
if ~isempty(fpath)
    fpath = [fpath '/'];
end
urlw = [fpath datdir fname '_or_frq.txt'];
% prepare results directories
if exist([fpath datdir],'dir') == 0
    mkdir([fpath datdir])
end
if strcmp(fig_save,'save')
    if exist([fpath pixdir],'dir') == 0
        mkdir([fpath pixdir])
    end
end

% delete any previous frequency count file with the same fname
if exist(urlw,'file') ~= 0
    delete(urlw);
end

% load target files list
L = read_imageslist(urlr);
z = size(L,1);

% display waitbar
h_wb_frq = waitbar(0,'');
h_wb_frq_pos = get(h_wb_frq,'OuterPosition');
set(h_wb_frq,'Name','The machine is working... Drink coffee.',...
    'OuterPosition',h_wb_frq_pos+[0 h_wb_frq_pos(4)*1.5 0 0])
id = 1;
waitbar(id/z, h_wb_frq, [num2str(id) ' / ' num2str(z) ' images'])
drawnow

% batch process images
for id = 1:z
    % read image
    urlk = cell2mat(L(id));
    I = imread(urlk);

    % preprocess image
    I = or_pre(I,preproc,stroke_type);

    % initializations ---
    % orientations resolution in degrees
    step_or = 1;
    % filter width in pixels
    filter_width = 30;

    % get orientations image
    I_or = orientation(I, filter_width, step_or, 'wbar');

    % get filename
    [~, fname] = fileparts(urlk);

    % save colorcoded orientation image
    if strcmp(fig_save,'save')

        % make image
        T = I_or;
        cval = 180/step_or;
        cmap = [0 0 0; hsv(cval)];
        m = max(max(T));
        cmap = cmap(1:m+2,:); % all images will have same colormap range
        T = uint8(T + step_or);
        % save image
        imwrite(T,cmap,[fpath pixdir '/' fname '_or_cmap.png'],'png')
    end
    
    
    % extract orientations frequencies raw data
    T = I_or(:);
    T(T<0) = [];
    T = hist(T,0:179);
    
    % save results to file
    % (if the file exists the new data is appended)
    if unicode2native(fname(1)) >= 48 && unicode2native(fname(1)) <= 57
        % avert filenames starting with a numerical
        % by adding an arbitrary prefix
        fname = ['IMG' fname];
    end
    s = regexprep(mat2str(T),'\s','\t');
    s = [fname '\t' s(2:end-1) '\n'];
    fid = fopen(urlw,'at');
    fprintf(fid, s);
    fclose(fid);

    % increment waitbar
    waitbar(id/z, h_wb_frq, [num2str(id) ' / ' num2str(z) ' images'])
end
% close waitbar
close(h_wb_frq)

