function I_or = orientation(I, filter_width, step, wbar)
% ORIENTATION. Computes the local orientation of an image.
%
% -------------
% INPUT
% -------------
% I - binary image containg either outlines or inlines
% filter_width - scalar
%     - width of the filter to be used on the image
% step - scalar
%     - angle in degrees, at which multiples the local script
%         orientation is computed
%     - default: 5
% wbar - {'wbar'} | 'nowbar'
%     - show progress as a waitbar or not
%     - optional
%
% -------------
% OUTPUT
% -------------
% I_or - orientation values map
%     - gives for each pixel in the input image its orientation
%         in degrees; values are real and evenly spaced by
%         the value given in the input variable 'step'
%     - negative values specify the background
% 
% -------------
% CREDITS
% -------------
% Vlad Atanasiu
% atanasiu@alum.mit.edu | http://alum.mit.edu/www/atanasiu/
% 2010.10.19 - creation

% parse inputs
if nargin < 3
    step = 5;
end
if nargin < 4
    wbar = false;
else
    if strcmp(wbar,'wbar') == 1
        wbar = true; % function will run with visual feedback
    else
        wbar = false;
    end
end

% prepare
[sz_y_I, sz_x_I] = size(I);
I_or = zeros(sz_y_I, sz_x_I);
I2_val = I_or;
I_FT = fft2(I);

% show progress
if wbar == true
    h = waitbar(0,''); % display waitbar
    set(h,'Name','The machine is working... Drink coffee.')
    drawnow
end

% filter image at given orientations
for theta = 0:step:180-step
    
    % make filter
    % F = gauss2_stripe(extent, width, theta, sigma, ...
    %     st_deviation, amplitude)
%     F = gauss2_stripe([sz_y_I, sz_x_I], 5, theta, .5, 5, 1);
    F = gauss2([filter_width filter_width], [2 .5], ...
        theta, 1, [-5 5], [sz_y_I, sz_x_I]);
    
    % convolve image and filter in frequency domain 
    F = I_FT.*fft2(F);
    F = real(fftshift(ifft2(F)));
    
    % retain only values along the contour
    F = F.*I;
    
    % get maximum orientation
    T = sign(F - I2_val);
    idx = find(T>0);
    if isempty(idx) ~= 1
        I2_val(idx) = F(idx);
        I_or(idx) = theta;
    end
    
    % increment waitbar
    if wbar == true
        waitbar( theta/(180-step), h,...
            [num2str(theta+1), ' / ', num2str(180-step+1), ...
            ' orientations'])
    end
end

% close waitbar
if wbar == true
    close(h);
end

% set background to negative value
I_or(I == 0) = -step;


